//
// ********************************************************************
// * DISCLAIMER                                                       *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.                                                             *
// *                                                                  *
// * By copying,  distributing  or modifying the Program (or any work *
// * based  on  the Program)  you indicate  your  acceptance of  this *
// * statement, and all its terms.                                    *
// ********************************************************************
//
//
// --------------------------------------------------------------
//      GEANT 4 - IcaG4 
//
// --------------------------------------------------------------

#include "G4RunManager.hh"
#include "G4UImanager.hh"
#include "G4UIterminal.hh"
#include "G4UItcsh.hh"

#ifdef G4UI_USE_ROOT
#include "G4UIRoot.hh"
#endif

#ifdef G4UI_USE_XM
#include "G4UIXm.hh"
#endif

#include "Randomize.hh"

#ifdef G4VIS_USE
#include "IcaVisManager.hh"
#endif

#include "IcaDetectorConstruction.hh"
#include "IcaPhysicsList.hh"
#include "IcaPrimaryGeneratorAction.hh"
#include "IcaRunAction.hh"
#include "IcaEventAction.hh"
#include "IcaStackingAction.hh"
#include "IcaTrackingAction.hh"
#include "IcaSteppingAction.hh"
#include "IcaSteppingVerbose.hh"
#include "IcaAnalysisManager.hh"

#include "IcaAnalysisExample.hh"

int main(int argc,char** argv) {
  
  // choose the Random engine
  HepRandom::setTheEngine(new RanecuEngine());
  
  // Ica Verbose output class
  G4VSteppingVerbose::SetInstance(new IcaSteppingVerbose());
  
  // construct the default run manager
  G4RunManager *runManager = new G4RunManager();
  
  // start ("default") histograming
  IcaAnalysisExample *analysis = new IcaAnalysisExample();
  
  // set mandatory initialization classes
  IcaDetectorConstruction *detector = new IcaDetectorConstruction();
  runManager->SetUserInitialization(detector);
  
  runManager->SetUserInitialization(new IcaPhysicsList(detector));
  
  G4UIsession *session = 0;
  
  if (argc == 1)   // define UI session for interactive mode
    {
#if defined (G4UI_USE_ROOT)
      // G4URoot is a ROOT based GUI
      session = new G4UIRoot(argc,argv);
#elif defined (G4UI_USE_XM)
      // G4UIXm is an Xm based GUI
      session = new G4UIXm(argc,argv);
#else
      // G4UIterminal is a (dumb) terminal
#ifdef G4UI_USE_TCSH
      session = new G4UIterminal(new G4UItcsh());
#else
      session = new G4UIterminal();
#endif
#endif
    }
  
#ifdef G4VIS_USE
  // visualization manager
  G4VisManager *visManager = new IcaVisManager();
  visManager->Initialize();
#endif
  
  // set user action classes
  runManager->SetUserAction(new IcaPrimaryGeneratorAction(detector));
  runManager->SetUserAction(new IcaRunAction());
  IcaEventAction *eventAction = new IcaEventAction();
  runManager->SetUserAction(eventAction);
  runManager->SetUserAction(new IcaStackingAction());
  runManager->SetUserAction(new IcaTrackingAction());
  runManager->SetUserAction(new IcaSteppingAction(eventAction));
  
  // get the pointer to the User Interface manager
  G4UImanager *UI = G4UImanager::GetUIpointer();
  
  // execute IcaG4.PreInit.mac (if present)
  UI->ApplyCommand("/control/execute IcaG4.PreInit.mac");
  
  // initialize G4 kernel
  runManager->Initialize();
  
  if (session != 0) // define UI session for interactive mode
    {
      UI->ApplyCommand("/control/execute vis.mac");
      // G4UIterminal is a (dumb) terminal
#if defined (G4UI_USE_ROOT) || defined (G4UI_USE_XM)
      // customize the G4UI menubar with a macro file
      UI->ApplyCommand("/control/execute vis/gui.mac");
#endif
      session->SessionStart();
      if ( (G4UImanager::GetUIpointer()) &&
	   (session == (G4UImanager::GetUIpointer())->GetSession()) )
	delete session;
    }
  else // batch mode
    {
      G4String command = "/control/execute ";
      G4String filename = argv[1];
      UI->ApplyCommand(command+filename);
    }
  
  // job termination
  
  if (analysis == IcaAnalysisManager::GetAnalysisManager())
    delete analysis;
  
#ifdef G4VIS_USE
  if (visManager == G4VVisManager::GetConcreteInstance())
    delete visManager;
#endif
  
  if (runManager == G4RunManager::GetRunManager())
    delete runManager;
  
  return 0;
}

// end of file IcaG4.cc by Jacek M. Holeczek
