//
// ********************************************************************
// * DISCLAIMER                                                       *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.                                                             *
// *                                                                  *
// * By copying,  distributing  or modifying the Program (or any work *
// * based  on  the Program)  you indicate  your  acceptance of  this *
// * statement, and all its terms.                                    *
// ********************************************************************
//

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

#include "IcaDetectorConstruction.hh"
#include "IcaDetectorConstructionMessenger.hh"

#include "IcaAnalysisManager.hh"

#include "G4Material.hh"
#include "G4Box.hh"
#include "G4LogicalVolume.hh"
#include "G4PVPlacement.hh"
#include "G4UniformMagField.hh"
#include "G4FieldManager.hh"
#include "G4TransportationManager.hh"

#include "G4RunManager.hh"

#include "G4VisAttributes.hh"
#include "G4Colour.hh"

#include "G4UserLimits.hh"

#include "G4UnitsTable.hh"
#include "G4ios.hh"

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

IcaDetectorConstruction::IcaDetectorConstruction()
:WorldSizeX (0.0), WorldSizeY (0.0), WorldSizeZ (0.0),
 pWorld   (0), lWorld (0),
 pBox     (0), lBox (0),
 dMaterial(0), wMaterial(0)
{
  WorldSizeX = 2.0*m;
  WorldSizeY = 2.0*m;
  WorldSizeZ = 10.0*m;
  
  LArAbsLength = 10.0 * WorldSizeX + 10.0 * WorldSizeY + 10.0 * WorldSizeZ;
  
  theUserLimitsForDetector = (G4UserLimits *)0;
  ResetUserLimits();
  
  // create the "/Simulation/" G4UIdirectory
  detectorMessenger = new IcaDetectorConstructionMessenger(this);
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

IcaDetectorConstruction::~IcaDetectorConstruction()
{
  if (theUserLimitsForDetector != 0) delete theUserLimitsForDetector;
  delete detectorMessenger;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

G4VPhysicalVolume* IcaDetectorConstruction::Construct()
{
  G4VPhysicalVolume *theWorldVolume = (G4VPhysicalVolume *)0;
  
  DefineMaterials();
  theWorldVolume = ConstructVolumes();
  
  // Histogramming
  if (gIcaAnalysisManager) gIcaAnalysisManager->Construct(theWorldVolume);
  
  return theWorldVolume;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

#include <cmath>

static const G4double k = 1.380658e-23 * joule/kelvin; // the Boltzmann constant
static const G4double dyne = 1.e-5 * newton; // compressibility uses this CGS unit
static const G4double LambdaE = 2.0 * 3.14159265358979323846 * 1.973269602e-16 * m * GeV;

// Properties of the Ar gas and LAr liquid

static const G4double ArRho  = 1.784 * kg/m3; // Ar (gas) mass density

static const G4double LArRho = 1.390 * g/cm3; // LAr mass density
static const G4double LArT = 87.0 * kelvin; // the actual temperature of LAr in detector
static const G4double LArKT = 2.18e-10 * cm2/dyne; // LAr isothermal compressibility

static const G4double LArScintillationYield = 10000.0 / MeV;
static const G4double LArScintillationResolutionScale = 1.0;
static const G4double LArScintillationFastTimeConstant = 6.0 * ns; // 27%
static const G4double LArScintillationSlowTimeConstant = 1.4 * microsecond; // 73%
static const G4double LArScintillationYieldRatio = 0.27; // fast component

// Calculates the dielectric constant of LAr from the Bideau-Sellmeier formula.
// See : A. Bideau-Mehu et al., "Measurement of refractive indices of Ne, Ar,
// Kr and Xe ...", J. Quant. Spectrosc. Radiat. Transfer, Vol. 25 (1981), 395

static G4double LArEpsilon(const G4double lambda)
{
  G4double epsilon;
  if (lambda <= (107.05 * nanometer)) return 1.0e4; // lambda MUST be > 107.05 nm
  epsilon = lambda / micrometer; // switch to micrometers
  epsilon = 1.0 / (epsilon * epsilon); // 1 / (lambda)^2
  epsilon = 1.2055e-2 * ( 0.2075 / (91.012 - epsilon) +
			  0.0415 / (87.892 - epsilon) +
			  4.3330 / (214.02 - epsilon) );
  epsilon *= (8./12.); // Bideau-Sellmeier -> Clausius-Mossotti
  epsilon *= (LArRho / ArRho); // density correction (Ar gas -> LAr liquid)
  if ((epsilon < 0.0) || (epsilon > 0.999999)) return 4.0e6;
  epsilon = (1.0 + 2.0 * epsilon) / (1.0 - epsilon); // solve Clausius-Mossotti
  return epsilon;
}

// Calculates the refractive index of LAr

static G4double LArRefIndex(const G4double lambda)
{
  return ( sqrt(LArEpsilon(lambda)) ); // square root of dielectric constant
}

// Calculates the Rayleigh scattering length using equations given in
// G. M. Seidel at al., "Rayleigh scattering in rare-gas liquids",
// arXiv:hep-ex/0111054 v2 22 Apr 2002

static G4double LArRayLength(const G4double lambda)
{
  G4double h;
  h = LArEpsilon(lambda);
  if (h < 1.00000001) h = 1.00000001; // just a precaution
  h = (h - 1.0) * (h + 2.0); // the "dielectric constant" dependance
  h *= h; // take the square
  h *= LArKT * LArT * k; // compressibility * temperature * Boltzmann constant
  h /= lambda * lambda * lambda * lambda; // (lambda)^4
  h *= 9.18704494231105429; // (2 * Pi / 3)^3
  if ( h < (1.0 / (10.0 * km)) ) h = 1.0 / (10.0 * km); // just a precaution
  if ( h > (1.0 / (0.1 * nanometer)) ) h = 1.0 / (0.1 * nanometer); // just a precaution
  return ( 1.0 / h );
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void IcaDetectorConstruction::DefineMaterials()
{ 
  //This function illustrates the possible ways to define materials
  
  G4String name, symbol;
  G4double a, z, density;            //a=mass of a mole, z=mean number of protons;
  G4int ncomponents;
  
  G4double temperature, pressure;
  
  //
  // vacuum
  //
  
  density = universe_mean_density; //from PhysicalConstants.h
  pressure = 3.e-18*pascal;
  temperature = 2.73*kelvin;
  G4Material* Vacuum = new G4Material(name="Vacuum", z=1., a=1.01*g/mole,
				      density,kStateGas,temperature,pressure);
  
  //
  // liquid argon
  //
  
  density = LArRho;
  a = 39.95*g/mole;
  G4Material* LAr = new G4Material(name="LiquidArgon", z=18., a, density);
  
  const G4int NUMENTRIES = 64;
  G4double LAr_PPCK[(NUMENTRIES)];
  G4double LAr_RIND[(NUMENTRIES)];
  G4double LAr_RAYL[(NUMENTRIES)];
  G4double LAr_ABSL[(NUMENTRIES)];
  
  G4double PPCKOVHighE = LambdaE / (109. * nanometer); // = 11.37 * eV
  G4double PPCKOVLowE = LambdaE / (700. * nanometer); // = 1.77 * eV
  G4double de = ((PPCKOVHighE - PPCKOVLowE) / ((G4double)(NUMENTRIES - 1)));
  G4double e;
  
  for (int i = 0; i < NUMENTRIES; i++)
    {
      e = PPCKOVLowE + ((G4double)i) * de;
      LAr_PPCK[i] = e;
      LAr_RIND[i] = LArRefIndex((LambdaE / e));
      LAr_RAYL[i] = LArRayLength((LambdaE / e));
      LAr_ABSL[i] = LArAbsLength;
    }
  
  G4double LAr_SCIN[3] = { 0.0, 1.0, 0.0 };
  G4double LAr_SCIN_ENE = LambdaE / (128. * nanometer);
  G4double LAr_SCPP[3] = { (LAr_SCIN_ENE - de / 100.0),
			   LAr_SCIN_ENE,
			   (LAr_SCIN_ENE + de / 100.0) };
  
  G4MaterialPropertiesTable *LArMPT = new G4MaterialPropertiesTable();
  LArMPT->AddProperty("RINDEX",        LAr_PPCK, LAr_RIND, NUMENTRIES);
  LArMPT->AddProperty("RAYLEIGH",      LAr_PPCK, LAr_RAYL, NUMENTRIES);
  LArMPT->AddProperty("ABSLENGTH",     LAr_PPCK, LAr_ABSL, NUMENTRIES);
  LArMPT->AddProperty("FASTCOMPONENT",
		      LAr_SCPP,
		      LAr_SCIN,
		      (sizeof(LAr_SCIN) / sizeof(G4double)));
  LArMPT->AddProperty("SLOWCOMPONENT",
		      LAr_SCPP,
		      LAr_SCIN,
		      (sizeof(LAr_SCIN) / sizeof(G4double)));
  LArMPT->AddConstProperty("SCINTILLATIONYIELD", LArScintillationYield);
  LArMPT->AddConstProperty("RESOLUTIONSCALE", LArScintillationResolutionScale);
  LArMPT->AddConstProperty("FASTTIMECONSTANT", LArScintillationFastTimeConstant);
  LArMPT->AddConstProperty("SLOWTIMECONSTANT", LArScintillationSlowTimeConstant);
  LArMPT->AddConstProperty("YIELDRATIO", LArScintillationYieldRatio);
  LAr->SetMaterialPropertiesTable(LArMPT);
  
#if 0
  //
  // aluminum
  //
  
  density = 2.700*g/cm3;
  a = 26.98154*g/mole;
  G4Material* Alumin = new G4Material(name="Aluminum", z=13., a, density);
#endif
  
  //
  // austenitic stainless steel (Medical Physics, Vol 25, No 10, Oct 1998)
  //
  
  G4Element* Fe = new G4Element
    (name="Iron", symbol="Fe", z=26., a=55.85*g/mole);
  G4Element* Cr  = new G4Element
    (name="Chromium", symbol="Cr", z=24., a=52.00*g/mole);
  G4Element* Ni  = new G4Element
    (name="Nickel", symbol="Ni", z=28., a=58.70*g/mole);
  G4Element* Mn  = new G4Element
    (name="Manganese", symbol="Mn", z=25., a=54.94*g/mole);
  G4Element* Si  = new G4Element
    (name="Silicon", symbol="Si", z=14., a=28.09*g/mole);
  
  G4Material* ASSteel = new G4Material
    (name="AusteniticStainlessSteel", density=8.02*g/cm3, ncomponents=5);
  ASSteel->AddElement(Fe, 0.68);
  ASSteel->AddElement(Cr, 0.19);
  ASSteel->AddElement(Ni, 0.10);
  ASSteel->AddElement(Mn, 0.02);
  ASSteel->AddElement(Si, 0.01);
  
#if 0
  G4Element* C  = new G4Element
    (name="Carbon", symbol="C", z=6., a=12.011*g/mole);
  G4Element* Co = new G4Element
    (name="Cobalt", symbol="Co", z=27., a=58.9332*g/mole);
#endif
  
  G4cout << *(G4Material::GetMaterialTable()) << G4endl;
  
  //default materials of the detector
  dMaterial = LAr;
  wMaterial = Vacuum;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

G4VPhysicalVolume* IcaDetectorConstruction::ConstructVolumes()
{    
  G4Box*
  sWorld = new G4Box("World",				//its name
                   WorldSizeX, WorldSizeY, WorldSizeZ);	//its dimensions
                 
  lWorld = new G4LogicalVolume(sWorld,			//its shape
                             wMaterial,			//its material
                             "World");		//its name
                                   
  pWorld = new G4PVPlacement(0,				//no rotation
  			   G4ThreeVector(),		//at (0,0,0)
                           "World",			//its name
                           lWorld,			//its logical volume
                           0,			//its mother  volume
                           false,			//no boolean operation
                           0);				//copy number
                           
  G4Box*
  sBox = new G4Box("T300",				//its name
                   WorldSizeX - 0.5*m,	//its dimensions
		   WorldSizeY - 0.5*m,
		   WorldSizeZ - 0.5*m);
                 
  lBox = new G4LogicalVolume(sBox,			//its shape
                             dMaterial,			//its material
                             "T300");		//its name

  // create UserLimits
  if (theUserLimitsForDetector != 0) delete theUserLimitsForDetector;
  theUserLimitsForDetector = new G4UserLimits( DBL_MAX,  //step max
					       DBL_MAX,  // track max
					       theMaxTimeCutsInDetector,
					       theMinEkineCutsInDetector,
					       theMinRangeCutsInDetector );

  // attach UserLimits
  UseUserLimits(fUseUserLimits);

  pBox = new G4PVPlacement(0,				//no rotation
  			   G4ThreeVector(),		//at (0,0,0)
                           "T300",			//its name
                           lBox,			//its logical volume
                           pWorld,			//its mother  volume
                           false,			//no boolean operation
                           0);				//copy number

  //
  // Visualization attributes
  //

  lWorld->SetVisAttributes (G4VisAttributes::Invisible);
  G4VisAttributes* visAtt= new G4VisAttributes(G4Colour(1.0,1.0,1.0));
  visAtt->SetVisibility(true);
  lBox->SetVisAttributes(visAtt);
  
  //
  //always return the root volume (physical World)
  //
  
  return pWorld;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void  IcaDetectorConstruction::ResetUserLimits()
{
  UseUserLimits( false );
  SetMaxTimeInDetector( 1.0 * ms );
  SetMinEkineInDetector( 1.0 * MeV );
  SetMinRangeInDetector( 1.0 * mm );
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void  IcaDetectorConstruction::UseUserLimits(G4bool isUse)
{
  fUseUserLimits = isUse;
  if (lBox) {
    if (!fUseUserLimits) lBox->SetUserLimits(0);
    else if (theUserLimitsForDetector!= 0) lBox->SetUserLimits(theUserLimitsForDetector);
  }
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void  IcaDetectorConstruction::SetMaxTimeInDetector(G4double value)
{
  theMaxTimeCutsInDetector = value;
  if (theUserLimitsForDetector != 0) {
    theUserLimitsForDetector->SetUserMaxTime(value);
  }
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void  IcaDetectorConstruction::SetMinEkineInDetector(G4double value)
{
  theMinEkineCutsInDetector = value;
  if (theUserLimitsForDetector != 0) {
    theUserLimitsForDetector->SetUserMinEkine(value);
  }
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void  IcaDetectorConstruction::SetMinRangeInDetector(G4double value)
{
  theMinRangeCutsInDetector = value;
  if (theUserLimitsForDetector != 0) {
    theUserLimitsForDetector->SetUserMinRange(value);
  }
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

// end of file IcaDetectorConstruction.cc by Jacek M. Holeczek
