//
// ********************************************************************
// * DISCLAIMER                                                       *
// *                                                                  *
// * The following disclaimer summarizes all the specific disclaimers *
// * of contributors to this software. The specific disclaimers,which *
// * govern, are listed with their locations in:                      *
// *   http://cern.ch/geant4/license                                  *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.                                                             *
// *                                                                  *
// * This  code  implementation is the  intellectual property  of the *
// * GEANT4 collaboration.                                            *
// * By copying,  distributing  or modifying the Program (or any work *
// * based  on  the Program)  you indicate  your  acceptance of  this *
// * statement, and all its terms.                                    *
// ********************************************************************
//
//
// $Id: IcaDetectorConstructionMessenger.cc,v 1.6 2002/01/09 17:24:12 ranjard Exp $
// GEANT4 tag $Name: geant4-04-00-patch-02 $
//
// 

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

#include "IcaDetectorConstructionMessenger.hh"
#include "IcaDetectorConstruction.hh"

#include "G4UIdirectory.hh"
#include "G4UIcmdWithoutParameter.hh"
#include "G4UIcmdWithABool.hh"
#include "G4UIcmdWithADoubleAndUnit.hh"
#include "globals.hh"
#include "iomanip"

#include "strstream"

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

IcaDetectorConstructionMessenger::IcaDetectorConstructionMessenger(
					IcaDetectorConstruction* IcaDet)
:IcaDetector(IcaDet)
{
  IcaSimDir = new G4UIdirectory("/Simulation/");
  IcaSimDir->SetGuidance("General purpose simulation control commands.");

  ResCmd = new G4UIcmdWithoutParameter("/Simulation/ResetUserLimitsInDetector",this);
  ResCmd->SetGuidance("Reset user limits in detector.");
  ResCmd->AvailableForStates(G4State_PreInit, G4State_Idle);

  SwitchCmd = new G4UIcmdWithABool("/Simulation/UseUserLimitsInDetector",this);
  SwitchCmd->SetGuidance("Switch user limits in detector on/off.");
  SwitchCmd->SetGuidance("  Default = true");
  SwitchCmd->SetParameterName("UseUserLimitsFlag",false);
  SwitchCmd->SetDefaultValue(true);
  SwitchCmd->AvailableForStates(G4State_PreInit, G4State_Idle);

  TmaxCmd = new G4UIcmdWithADoubleAndUnit("/Simulation/SetMaxTimeInDetector",this);
  TmaxCmd->SetGuidance("Set maximum time in detector.");
  TmaxCmd->SetParameterName("Tmax",false,false);
  TmaxCmd->SetDefaultUnit("ns");
  TmaxCmd->SetUnitCategory("Time");
  TmaxCmd->AvailableForStates(G4State_PreInit, G4State_Idle);

  EminCmd = new G4UIcmdWithADoubleAndUnit("/Simulation/SetMinEkinInDetector",this);
  EminCmd->SetGuidance("Set minimum kinetic energy in detector.");
  EminCmd->SetParameterName("Emin",false,false);
  EminCmd->SetDefaultUnit("MeV");
  EminCmd->SetUnitCategory("Energy");
  EminCmd->AvailableForStates(G4State_PreInit, G4State_Idle);

  RminCmd = new G4UIcmdWithADoubleAndUnit("/Simulation/SetMinRangeInDetector",this);
  RminCmd->SetGuidance("Set minimum range in detector.");
  RminCmd->SetParameterName("Rmin",false,false);
  RminCmd->SetDefaultUnit("mm");
  RminCmd->SetUnitCategory("Length");
  RminCmd->AvailableForStates(G4State_PreInit, G4State_Idle);

  ALenCmd = new G4UIcmdWithADoubleAndUnit("/Simulation/SetOpAbsLengthInLAr",this);
  ALenCmd->SetGuidance("Set optical photon absorption length in LAr.");
  ALenCmd->SetParameterName("LArAbsLength",false,false);
  ALenCmd->SetDefaultUnit("m");
  ALenCmd->SetUnitCategory("Length");
  ALenCmd->AvailableForStates(G4State_PreInit);
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

IcaDetectorConstructionMessenger::~IcaDetectorConstructionMessenger()
{
  delete ResCmd;
  delete SwitchCmd;
  delete TmaxCmd;
  delete RminCmd;
  delete EminCmd;
  
  delete ALenCmd;
  
  delete IcaSimDir;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void IcaDetectorConstructionMessenger::SetNewValue(G4UIcommand * command,G4String newValue)
{
  if( command == ResCmd ) {
    IcaDetector->ResetUserLimits();

  } else if( command == SwitchCmd ) {
    G4int vl;
    const char* t = newValue;
    std::istrstream is((char*)t);
    is >> vl;
    IcaDetector->UseUserLimits(vl!=0);

  } else if( command == TmaxCmd ) {
    IcaDetector->SetMaxTimeInDetector(TmaxCmd->GetNewDoubleValue(newValue));

  } else if( command == EminCmd ) {
    IcaDetector->SetMinEkineInDetector(EminCmd->GetNewDoubleValue(newValue));

  } else if( command == RminCmd ) {
    IcaDetector->SetMinRangeInDetector(RminCmd->GetNewDoubleValue(newValue));

  } else if( command == ALenCmd ) {
    IcaDetector->SetLArAbsLength(ALenCmd->GetNewDoubleValue(newValue));
  }
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

G4String IcaDetectorConstructionMessenger::GetCurrentValue(G4UIcommand * command)
{
  G4String returnValue('\0');
  char line[255];
  std::ostrstream os(line,255);

  if( command == ResCmd ) {
    returnValue = "***NULL***";

  } else if( command == SwitchCmd ) { 
    if ( IcaDetector->IsUseUserLimits() ) returnValue = "1";
    else returnValue = "0";

  } else if( command == TmaxCmd ) {
    os <<  IcaDetector->GetMaxTimeInDetector()/ns << " ns" << '\0';
    returnValue = G4String(line);

  } else if( command == EminCmd ) {
    os <<  IcaDetector->GetMinEkineInDetector()/MeV << " MeV" << '\0';
    returnValue = G4String(line);

  } else if( command == RminCmd ) {
    os <<  IcaDetector->GetMinRangeInDetector()/mm << " mm" << '\0';
    returnValue = G4String(line);

  } else if( command == ALenCmd ) {
    os <<  IcaDetector->GetLArAbsLength()/m << " m" << '\0';
    returnValue = G4String(line);
  }

  return returnValue;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

// end of file IcaDetectorConstructionMessenger.hh by Jacek M. Holeczek
